==> audiochain-1.1.5-src.tgz: org/audiochain/ui/gui/mixer/ChooseAudioDeviceDialog.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static org.audiochain.ui.gui.CommonComponentFactory.*;
import static org.audiochain.ui.gui.Dialog.*;

import java.util.Collection;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JScrollPane;

import org.audiochain.model.AudioDevice;
import org.audiochain.model.AudioDeviceFactory;
import org.audiochain.model.AudioDeviceFactory.AudioDeviceIdentifier;
import org.audiochain.model.AudioDeviceRegistry;
import org.audiochain.model.ChainableAudioDevice;
import org.audiochain.model.SourceAudioDevice;
import org.audiochain.ui.gui.Dialog;

public class ChooseAudioDeviceDialog {

	public static ChainableAudioDevice selectChainableAudioDevice(Object caller) {
		try {
			AudioDeviceRegistry registry = AudioDeviceRegistry.getInstance();
			Collection<Class<ChainableAudioDevice>> chainableAudioDeviceClasses = registry.getChainableAudioDeviceClasses();
			Vector<ListEntry> entries = createEntries(caller, chainableAudioDeviceClasses);
			Collection<AudioDeviceFactory> chainableAudioDeviceFactories = registry.getChainableAudioDeviceFactories();
			for (AudioDeviceFactory chainableAudioDeviceFactory : chainableAudioDeviceFactories) {
				Collection<AudioDeviceIdentifier> audioDeviceIdentifiers = chainableAudioDeviceFactory.getAudioDeviceIdentifiers();
				for (AudioDeviceIdentifier identifier : audioDeviceIdentifiers) {
					if (!identifier.isSourceAudioDevice()) {
						ListEntry entry = new ListEntry();
						entry.deviceIdentifier = identifier;
						entry.label = identifier.getName();
						entry.additionalObject = chainableAudioDeviceFactory;
						entries.add(entry);
					}
				}
			}
			return selectAndCreateInstance(caller, entries);
		} catch (Exception e) {
			e.printStackTrace();
			Dialog.message(caller, "Error while loading audio devices", e);
		}
		return null;
	}

	public static SourceAudioDevice selectSourceAudioDevice(Object caller) {
		try {
			Collection<Class<SourceAudioDevice>> sourceAudioDeviceClasses = AudioDeviceRegistry.getInstance().getSourceAudioDeviceClasses();
			Vector<ListEntry> entries = createEntries(caller, sourceAudioDeviceClasses);
			return selectAndCreateInstance(caller, entries);
		} catch (Exception e) {
			e.printStackTrace();
			Dialog.message(caller, "Error while loading audio devices", e);
		}
		return null;
	}

	public static <E extends AudioDevice> Vector<ListEntry> createEntries(Object caller, Collection<Class<E>> classes) {
		Vector<ListEntry> ret = new Vector<ListEntry>();
		for (Class<? extends AudioDevice> klass : classes) {
			ListEntry entry = new ListEntry();
			String name = createInstance(caller, klass).getName();
			entry.deviceIdentifier = name;
			entry.label = name;
			entry.additionalObject = klass;
			ret.add(entry);
		}
		return ret;
	}

	public static <E extends AudioDevice> E selectAndCreateInstance(Object caller, Vector<ListEntry> entries) {
		JButton addButton = new JButton("Select Audio Device");
		JList list = new JList();
		list.setListData(entries);
		JScrollPane scroll = createScrollPane(list);
		if (isButtonPressed(caller, "Choose Audio Device", scroll, addButton)) {
			ListEntry entry = (ListEntry) list.getSelectedValue();
			if (entry.additionalObject instanceof AudioDeviceFactory) {
				AudioDeviceFactory factory = (AudioDeviceFactory) entry.additionalObject;
				@SuppressWarnings("unchecked")
				E ret = (E) factory.createAudioDevice((AudioDeviceIdentifier) entry.deviceIdentifier);
				return ret;
			} else {
				@SuppressWarnings("unchecked")
				Class<E> klass = (Class<E>) entry.additionalObject;
				return createInstance(caller, klass);
			}
		}
		return null;
	}

	private static <E> E createInstance(Object caller, Class<E> klass) {
		if (klass == null) {
			return null;
		}
		try {
			E audioDevice = klass.newInstance();
			return audioDevice;
		} catch (InstantiationException e) {
			e.printStackTrace();
			Dialog.message(caller, "Instantiate Audio Device", e);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			Dialog.message(caller, "Access Audio Device", e);
		}
		return null;
	}
}

class ListEntry {
	Object deviceIdentifier;
	String label;
	// place to remember an associated object used for creation
	Object additionalObject;

	@Override
	public String toString() {
		return label;
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/gui/mixer/ChooseAudioDeviceDialog.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static org.audiochain.ui.gui.CommonComponentFactory.createScrollPane;
import static org.audiochain.ui.gui.Dialog.isButtonPressed;

import java.util.Collection;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JScrollPane;

import org.audiochain.model.AudioDevice;
import org.audiochain.model.AudioDeviceFactory;
import org.audiochain.model.AudioDeviceFactory.AudioDeviceIdentifier;
import org.audiochain.model.AudioDeviceRegistry;
import org.audiochain.model.ChainableAudioDevice;
import org.audiochain.model.SourceAudioDevice;
import org.audiochain.ui.gui.Dialog;

public class ChooseAudioDeviceDialog {

	public static ChainableAudioDevice selectChainableAudioDevice(Object caller) {
		try {
			AudioDeviceRegistry registry = AudioDeviceRegistry.getInstance();
			Collection<Class<ChainableAudioDevice>> chainableAudioDeviceClasses = registry.getChainableAudioDeviceClasses();
			Vector<ListEntry> entries = createEntries(caller, chainableAudioDeviceClasses);
			Collection<AudioDeviceFactory> chainableAudioDeviceFactories = registry.getChainableAudioDeviceFactories();
			for (AudioDeviceFactory chainableAudioDeviceFactory : chainableAudioDeviceFactories) {
				Collection<AudioDeviceIdentifier> audioDeviceIdentifiers = chainableAudioDeviceFactory.getAudioDeviceIdentifiers();
				for (AudioDeviceIdentifier identifier : audioDeviceIdentifiers) {
					if (!identifier.isSourceAudioDevice()) {
						ListEntry entry = new ListEntry();
						entry.deviceIdentifier = identifier;
						entry.label = identifier.getName();
						entry.additionalObject = chainableAudioDeviceFactory;
						entries.add(entry);
					}
				}
			}
			return selectAndCreateInstance(caller, entries);
		} catch (Exception e) {
			e.printStackTrace();
			Dialog.message(caller, "Error while loading audio devices", e);
		}
		return null;
	}

	public static SourceAudioDevice selectSourceAudioDevice(Object caller) {
		try {
			Collection<Class<SourceAudioDevice>> sourceAudioDeviceClasses = AudioDeviceRegistry.getInstance().getSourceAudioDeviceClasses();
			Vector<ListEntry> entries = createEntries(caller, sourceAudioDeviceClasses);
			return selectAndCreateInstance(caller, entries);
		} catch (Exception e) {
			e.printStackTrace();
			Dialog.message(caller, "Error while loading audio devices", e);
		}
		return null;
	}

	public static <E extends AudioDevice> Vector<ListEntry> createEntries(Object caller, Collection<Class<E>> classes) {
		Vector<ListEntry> ret = new Vector<ListEntry>();
		for (Class<? extends AudioDevice> klass : classes) {
			ListEntry entry = new ListEntry();
			String name = createInstance(caller, klass).getName();
			entry.deviceIdentifier = name;
			entry.label = name;
			entry.additionalObject = klass;
			ret.add(entry);
		}
		return ret;
	}

	public static <E extends AudioDevice> E selectAndCreateInstance(Object caller, Vector<ListEntry> entries) {
		JButton addButton = new JButton("Select Audio Device");
		JList<ListEntry> list = new JList<ListEntry>();
		list.setListData(entries);
		JScrollPane scroll = createScrollPane(list);
		if (isButtonPressed(caller, "Choose Audio Device", scroll, addButton)) {
			ListEntry entry = list.getSelectedValue();
			if (entry.additionalObject instanceof AudioDeviceFactory) {
				AudioDeviceFactory factory = (AudioDeviceFactory) entry.additionalObject;
				@SuppressWarnings("unchecked")
				E ret = (E) factory.createAudioDevice((AudioDeviceIdentifier) entry.deviceIdentifier);
				return ret;
			} else {
				@SuppressWarnings("unchecked")
				Class<E> klass = (Class<E>) entry.additionalObject;
				return createInstance(caller, klass);
			}
		}
		return null;
	}

	private static <E> E createInstance(Object caller, Class<E> klass) {
		if (klass == null) {
			return null;
		}
		try {
			E audioDevice = klass.newInstance();
			return audioDevice;
		} catch (InstantiationException e) {
			e.printStackTrace();
			Dialog.message(caller, "Instantiate Audio Device", e);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			Dialog.message(caller, "Access Audio Device", e);
		}
		return null;
	}
}

class ListEntry {
	Object deviceIdentifier;
	String label;
	// place to remember an associated object used for creation
	Object additionalObject;

	@Override
	public String toString() {
		return label;
	}
}
